part of nodejs_interop;

/// A Node.js platform implementation.
class NodejsPlatform extends Platform {

  /// Creates a new Node.js platform.
  const NodejsPlatform();

  /// The environment for this process.
  /// It is an unmodifiable map whose content is retrieved from the operating system.
  ///
  /// Environment variables on Windows are case-insensitive. The keys of the map returned on Windows are therefore case-insensitive.
  /// On other platforms the keys of the returned map are case-sensitive.
  @override
  Map<String, String> get environment {
    var env = convertToDart(process.env);
    return new UnmodifiableMapView(isWindows ? new EqualityMap.from(const CaseInsensitiveEquality(), env) : env);
  }

  /// The path of the executable used to run the script in this isolate.
  ///
  /// The path returned is the literal path used to run the script.
  /// To get the absolute path to the resolved executable use [resolvedExecutable].
  @override
  String get executable => process.argv0;

  /// The flags passed to the executable used to run the script in this isolate.
  @override
  List<String> get executableArguments => process.execArgv;

  /// The name of the current locale.
  @override
  String get localeName {
    var env = environment;
    for (var key in ['LC_ALL', 'LC_MESSAGES', 'LANG', 'LANGUAGE']) {
      if (env.containsKey(key)) return env[key].replaceFirst(new RegExp('[.:].*'), '');
    }

    throw new UnimplementedError('Not implemented by the Node.js runtime.');
  }

  /// The local hostname for the system.
  @override
  String get localHostname => (require('os') as OsModule).hostname();

  /// The number of processors of the machine.
  @override
  int get numberOfProcessors => (require('os') as OsModule).cpus().length;

  /// A string representing the operating system.
  @override
  String get operatingSystem {
    switch (process.platform) {
      case 'darwin': return 'macos';
      case 'win32': return 'windows';
      default: return 'linux';
    }
  }

  /// The value of the `--packages` flag passed to the executable used to run the script in this isolate.
  @override
  String get packageConfig => throw new UnsupportedError('Not supported by the Node.js runtime.');

  /// The value of the `--package-root` flag passed to the executable used to run the script in this isolate.
  @override
  String get packageRoot => throw new UnsupportedError('Not supported by the Node.js runtime.');

  /// The path separator used by the operating system to separate components in file paths.
  @override
  String get pathSeparator => (require('path') as PathModule).sep;

  /// The path of the executable used to run the script in this isolate after it has been resolved by the OS.
  /// This is the absolute path, with all symlinks resolved, to the executable used to run the script.
  @override
  String get resolvedExecutable => process.execPath;

  /// The absolute URI of the script being run in this isolate.
  @override
  Uri get script => new Uri.file(filename, windows: isWindows);

  /// When `stdin` is connected to a terminal, whether ANSI codes are supported.
  @override
  bool get stdinSupportsAnsi => throw new UnimplementedError('Not implemented by the Node.js runtime.');

  /// When `stdout` is connected to a terminal, whether ANSI codes are supported.
  @override
  bool get stdoutSupportsAnsi => throw new UnimplementedError('Not implemented by the Node.js runtime.');

  /// The version of the current Node.js runtime.
  /// The value is a [semantic versioning](http://semver.org) string representing the version of the current Node.js runtime.
  @override
  String get version => process.version.substring(1);
}
