import 'dart:js';
import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:test/test.dart';

/// Tests the features of the `AssertModule` class.
void main() => group('AssertModule', () {
  final AssertModule _assert = require('assert');

  group('.deepEqual()', () {
    test('should not throw an assertion error if the actual and expected value are deeply equal', () {
      var map = new JsObject.jsify({'foo': 123});
      expect(() => _assert.deepEqual(map, new JsObject.jsify({'foo': '123'})), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not deeply equal', () {
      var map = new JsObject.jsify({'foo': 'bar'});
      expect(() => _assert.deepEqual(map, new JsObject.jsify({'foo': 'qux'})), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.deepStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are deeply and strictly equal', () {
      var map = new JsObject.jsify({'foo': '123'});
      expect(() => _assert.deepStrictEqual(map, new JsObject.jsify({'foo': '123'})), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not deeply and strictly equal', () {
      var map = new JsObject.jsify({'foo': '123'});
      expect(() => _assert.deepStrictEqual(map, new JsObject.jsify({'foo': 123})), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.doesNotThrow()', () {
    test('should not throw an assertion error if the block does not throw an error', () {
      var block = allowInterop(() => null);
      expect(() => _assert.doesNotThrow(block), returnsNormally);
    });

    test('should throw an assertion error if the block throws an error', () {
      var block = allowInterop(() => throw new Error());
      expect(() => _assert.doesNotThrow(block), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.equal()', () {
    test('should not throw an assertion error if the actual and expected value are equal', () {
      expect(() => _assert.equal(123, '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not equal', () {
      expect(() => _assert.equal('foo', 'bar'), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.fail()', () {
    test('should throw an assertion error', () {
      expect(_assert.fail, throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.ifError()', () {
    test('should not throw an assertion error if the value is falsy', () {
      expect(() => _assert.ifError(null), returnsNormally);
    });

    test('should throw an assertion error if the value is truthy', () {
      expect(() => _assert.ifError(new JsError()), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.notDeepEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not deeply equal', () {
      var map = new JsObject.jsify({'foo': 'bar'});
      expect(() => _assert.notDeepEqual(map, new JsObject.jsify({'foo': 'qux'})), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are deeply equal', () {
      var map = new JsObject.jsify({'foo': 123});
      expect(() => _assert.notDeepEqual(map, new JsObject.jsify({'foo': '123'})), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.notDeepStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not deeply and strictly equal', () {
      var map = new JsObject.jsify({'foo': '123'});
      expect(() => _assert.notDeepStrictEqual(map, new JsObject.jsify({'foo': 123})), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are deeply and strictly equal', () {
      var map = new JsObject.jsify({'foo': '123'});
      expect(() => _assert.notDeepStrictEqual(map, new JsObject.jsify({'foo': '123'})), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.notEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not equal', () {
      expect(() => _assert.notEqual('foo', 'bar'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are equal', () {
      expect(() => _assert.notEqual(123, '123'), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.notStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not strictly equal', () {
      expect(() => _assert.notStrictEqual(123, '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are strictly equal', () {
      expect(() => _assert.notStrictEqual('123', '123'), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.ok()', () {
    test('should not throw an assertion error if the value is truthy', () {
      expect(() => _assert.ok(true), returnsNormally);
    });

    test('should throw an assertion error if the value is falsy', () {
      expect(() => _assert.ok(false), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.strictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are strictly equal', () {
      expect(() => _assert.strictEqual('123', '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not strictly equal', () {
      expect(() => _assert.strictEqual(123, '123'), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });

  group('.throws()', () {
    test('should not throw an assertion error if the block throws an error', () {
      var block = allowInterop(() => throw new Error());
      expect(() => _assert.ok(block), returnsNormally);
    });

    test('should throw an assertion error if the block does not throw an error', () {
      var block = allowInterop(() => null);
      expect(() => _assert.throws(block), throwsA(const isInstanceOf<JsAssertionError>()));
    });
  });
});
