import 'dart:js';
import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:test/test.dart';

/// Tests the features of the `EventEmitter` class.
void main() => group('EventEmitter', () {
  var noop = allowInterop(() => null);
  require('events');

  group('.defaultMaxListeners', () {
    test('should get or set the default maximum number of listeners', () {
      expect(EventEmitter.defaultMaxListeners, equals(10));
      EventEmitter.defaultMaxListeners = 25;
      expect(EventEmitter.defaultMaxListeners, equals(25));
      EventEmitter.defaultMaxListeners = 10;
    });
  });

  group('.emit()', () {
    test('should call each of the listeners registered for a given event', () {
      var count = 0;
      var listener = allowInterop(() => count++);

      var emitter = new EventEmitter()..emit('event');
      expect(count, equals(0));
      emitter..on('event', listener)..emit('event');
      expect(count, equals(1));
      emitter.emit('event');
      expect(count, equals(2));
      emitter..on('event', listener)..emit('event');
      expect(count, equals(4));
    });
  });

  group('.eventNames()', () {
    test('should return an empty list if the emitter has no listener', () {
      expect(new EventEmitter().eventNames(), isEmpty);
    });

    test('should return the list of events for which the emitter has registered listeners', () {
      var emitter = new EventEmitter()..on('event1', noop);
      expect(emitter.eventNames(), unorderedEquals(['event1']));
      emitter.on('event2', noop);
      expect(emitter.eventNames(), unorderedEquals(['event1', 'event2']));
    });
  });

  group('.getMaxListeners()/.setMaxListeners()', () {
    test('should get and set the maximum number of listeners', () {
      var emitter = new EventEmitter();
      expect(emitter.getMaxListeners(), equals(EventEmitter.defaultMaxListeners));
      emitter.setMaxListeners(100);
      expect(EventEmitter.defaultMaxListeners, isNot(equals(100)));
      expect(emitter.getMaxListeners(), equals(100));
    });
  });

  group('.listenerCount()', () {
    test('should return the number of listeners listening to a given event', () {
      var emitter = new EventEmitter();
      expect(emitter.listenerCount('event'), equals(0));
      emitter.on('event', noop);
      expect(emitter.listenerCount('event'), equals(1));
      emitter.on('event', noop);
      expect(emitter.listenerCount('event'), equals(2));
    });
  });

  group('.listeners()', () {
    test('should return the list of listeners listening to a given event', () {
      var emitter = new EventEmitter();
      expect(emitter.listeners('event'), isEmpty);
      emitter.on('event', noop);
      expect(emitter.listeners('event'), orderedEquals([noop]));
      emitter.on('event', noop);
      expect(emitter.listeners('event'), orderedEquals([noop, noop]));
    });
  });

  group('.once()', () {
    test('should only invoke once its listener', () {
      var count = 0;
      var listener = allowInterop(() => count++);

      var emitter = new EventEmitter()..emit('event');
      expect(count, equals(0));
      emitter..once('event', listener)..emit('event');
      expect(count, equals(1));
      emitter.emit('event');
      expect(count, equals(1));
    });
  });

  group('.prependListener()', () {
    test('should add the listener function to the beginning of the listener list', () {
      var emitter = new EventEmitter();
      var listener = allowInterop(() => null);
      expect(emitter.listeners('event'), isEmpty);

      emitter.prependListener('event', noop);
      expect(emitter.listeners('event'), orderedEquals([noop]));
      emitter.prependListener('event', listener);
      expect(emitter.listeners('event'), orderedEquals([listener, noop]));
    });
  });

  group('.prependOnceListener()', () {
    test('should add the listener function to the beginning of the listener list and only invoke it once', () {
      var count = 0;
      var listener = allowInterop(() => count++);

      var emitter = new EventEmitter()
        ..on('event', noop)
        ..prependOnceListener('event', listener)
        ..emit('event');

      expect(count, equals(1));
      emitter.emit('event');
      expect(count, equals(1));
    });
  });

  group('.removeAllListeners()', () {
    test('should remove all the listeners', () {
      var emitter = new EventEmitter()
        ..on('event1', noop)..on('event2', noop)
        ..on('event1', noop)..on('event2', noop);

      expect(emitter.listenerCount('event1'), equals(2));
      expect(emitter.listenerCount('event2'), equals(2));
      emitter.removeAllListeners();
      expect(emitter.listenerCount('event1'), equals(0));
      expect(emitter.listenerCount('event2'), equals(0));
    });

    test('should remove all the listeners of a given event', () {
      var emitter = new EventEmitter()
        ..on('event1', noop)..on('event2', noop)
        ..on('event1', noop)..on('event2', noop);

      expect(emitter.listenerCount('event1'), equals(2));
      expect(emitter.listenerCount('event2'), equals(2));
      emitter.removeAllListeners('event1');
      expect(emitter.listenerCount('event1'), equals(0));
      expect(emitter.listenerCount('event2'), equals(2));
    });
  });

  group('.removeListener()', () {
    test('should remove the first occurence of the specified listener', () {
      var emitter = new EventEmitter()..on('event', noop)..on('event', noop);
      expect(emitter.listenerCount('event'), equals(2));
      emitter.removeListener('event', noop);
      expect(emitter.listenerCount('event'), equals(1));
      emitter.removeListener('event', noop);
      expect(emitter.listenerCount('event'), equals(0));
    });
  });
});
