import 'dart:js';
import 'package:nodejs_interop/nodejs_interop.dart';
import 'package:test/test.dart';

/// Tests the features of the `PathModule` class.
void main() => group('PathModule', () {
  final isWindows = context['process']['platform'] == 'win32';
  final PathModule path = require('path');

  group('.delimiter', () {
    test('should be `;` on Windows and `:` on other platforms', () {
      expect(path.delimiter, equals(isWindows ? ';' : ':'));
    });
  });

  group('.sep', () {
    test('should be `\\` on Windows and `/` on other platforms', () {
      expect(path.sep, equals(isWindows ? r'\' : '/'));
    });
  });

  group('.basename()', () {
    test('should return the file name from the specified path', () {
      expect(path.basename('/foo/bar/baz/asdf/quux.html'), equals('quux.html'));
    });

    test('should return the file name without the specified extension', () {
      expect(path.basename('/foo/bar/baz/asdf/quux.html', '.html'), equals('quux'));
      expect(path.basename('/foo/bar/baz/asdf/quux.html', '.txt'), equals('quux.html'));
    });
  });

  group('.dirname()', () {
    test('should return the directory name from the specified path', () {
      expect(path.dirname('path/to/foo.dart'), equals('path/to'));
      expect(path.dirname('path/to/'), equals('path'));
    });
  });

  group('.extname()', () {
    test('should return the file extension from the specified path', () {
      expect(path.extname('index.html'), equals('.html'));
      expect(path.extname('index.coffee.md'), equals('.md'));
      expect(path.extname('index.'), equals('.'));
      expect(path.extname('index'), isEmpty);
      expect(path.extname('.index'), isEmpty);
    });
  });

  group('.format()', () {
    test('should return a path formatted according to the specified object', () {
      expect(path.format(new Path(base: 'file.txt', dir: '/home/user/dir', root: '/ignored')), equals('/home/user/dir/file.txt'));
      expect(path.format(new Path(base: 'file.txt', ext: 'ignored', root: '/')), equals('/file.txt'));
      expect(path.format(new Path(ext: '.txt', name: 'file', root: '/')), equals('/file.txt'));
    });
  });

  group('.isAbsolute()', () {
    test('should return whether the specified path is a POSIX absolute one', () {
      expect(path.isAbsolute('/foo/bar'), isTrue);
      expect(path.isAbsolute('/baz/..'), isTrue);
      expect(path.isAbsolute('qux/'), isFalse);
      expect(path.isAbsolute('.'), isFalse);
    }, testOn: 'posix');

    test('should return whether the specified path is a Windows absolute one', () {
      expect(path.isAbsolute('//server'), isTrue);
      expect(path.isAbsolute(r'\\server'), isTrue);
      expect(path.isAbsolute('C:/foo/..'), isTrue);
      expect(path.isAbsolute(r'C:\foo\..'), isTrue);
      expect(path.isAbsolute(r'bar\baz'), isFalse);
      expect(path.isAbsolute('bar/baz'), isFalse);
      expect(path.isAbsolute('.'), isFalse);
    }, testOn: 'windows');
  });

  group('.join()', () {
    test('should join path segments using the path separator for the current platform', () {
      expect(path.join('/foo', 'bar', 'baz/asdf', 'quux', '..'), equals('/foo/bar/baz/asdf'));
    });
  });

  group('.normalize()', () {
    test('should normalize POSIX paths', () {
      expect(path.normalize('/foo/bar//baz/asdf/quux/..'), equals('/foo/bar/baz/asdf'));
    }, testOn: 'posix');

    test('should normalize Windows paths', () {
      expect(path.normalize(r'C:\temp\\foo\bar\..\'), equals(r'C:\temp\foo\bar'));
      expect(path.normalize(r'C:////temp\\/\/\/foo/bar'), equals(r'C:\temp\foo\bar'));
    }, testOn: 'windows');
  });

  group('.parse()', () {
    test('should parse a POSIX path into an object', () {
      var object = path.parse('/home/user/dir/file.txt');
      expect(object.root, equals('/'));
      expect(object.dir, equals('/home/user/dir'));
      expect(object.base, equals('file'));
      expect(object.ext, equals('.txt'));
      expect(object.name, equals('file.txt'));
    }, testOn: 'posix');

    test('should parse a Windows path into an object', () {
      var object = path.parse(r'C:\path\dir\file.txt');
      expect(object.root, equals(r'C:\'));
      expect(object.dir, equals(r'C:\path\dir'));
      expect(object.base, equals('file'));
      expect(object.ext, equals('.txt'));
      expect(object.name, equals('file.txt'));
    }, testOn: 'windows');
  });

  group('.relative()', () {
    test('should convert a POSIX path to an equivalent relative one', () {
      expect(path.relative('/data/orandea/test/aaa', '/data/orandea/impl/bbb'), equals('../../impl/bbb'));
    }, testOn: 'posix');

    test('should convert a Windows path to an equivalent relative one', () {
      expect(path.relative(r'C:\orandea\test\aaa', r'C:\orandea\impl\bbb'), equals(r'..\..\impl\bbb'));
    }, testOn: 'windows');
  });

  group('.resolve()', () {
    test('should resolve the specified paths', () {
      expect(path.resolve('/foo/bar', './baz'), equals('/foo/bar/baz'));
      expect(path.resolve('/foo/bar', '/tmp/file/'), equals('/tmp/file'));
      expect(path.resolve('wwwroot', 'static_files/png/', '../gif/image.gif'), endsWith('/wwwroot/static_files/gif/image.gif'));
    });
  });

  group('.toNamespacedPath()', () {
    test('should return the specified path on POSIX', () {
      expect(path.toNamespacedPath('/foo/bar'), equals('/foo/bar'));
    }, testOn: 'posix');

    test('should return a namespace-prefixed path on Windows', () {
      expect(path.toNamespacedPath(r'C:\foo\bar'), equals(r'\\?\C:\foo\bar'));
    }, testOn: 'windows');
  });
});
